package com.buzzvil.buzz_booster

import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import com.buzzvil.booster.external.BuzzBooster
import com.buzzvil.booster.external.BuzzBoosterConfig
import com.buzzvil.booster.external.campaign.CampaignType
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import android.app.Activity
import android.content.Context

/** BuzzBoosterPlugin */
class BuzzBoosterPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel
  private lateinit var context : Context
  private var activity: Activity? = null

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    context = flutterPluginBinding.applicationContext
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "buzz_booster")
    channel.setMethodCallHandler(this)

    flutterPluginBinding
        .platformViewRegistry
        .registerViewFactory("CampaignFloatingActionButton", FLCampaignFloatingActionButtonFactory())
        
    flutterPluginBinding
        .platformViewRegistry
        .registerViewFactory("CampaignEntryView", FLCampaignEntryViewFactory())
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    if (call.method == "init") {
      val appKey = call.argument("androidAppKey") as String?
      if (appKey != null) {
        val config = BuzzBoosterConfig(appKey!!)
        BuzzBooster.init(context, config)
        result.success(null)
      } else {
        result.error("400", "AndroidAppKey is required", null)
      }
    } else if (call.method == "setUserId") {
      val userId = call.argument("userId") as String?
      BuzzBooster.setUserId(userId)
      result.success(null)
    } else if (call.method == "sendEvent") {
      val eventName = call.argument("eventName") as String?
      if (eventName != null) {
        val eventValues = call.argument("eventValues") as Map<String, String>?
        if (eventValues != null) {
          BuzzBooster.getInstance().sendEvent(eventName, eventValues)
        } else {
          BuzzBooster.getInstance().sendEvent(eventName)
        }
        result.success(null)
      } else {
        result.error("400", "EventName is required", null)
      }
    } else if (call.method == "showInAppMessage") {
      if (activity != null) { 
        BuzzBooster.getInstance().showInAppMessage(activity!!)
      }
      result.success(null)
    } else if (call.method == "showCampaign") {
      if (activity != null) { 
        BuzzBooster.getInstance().showCampaign(activity!!)
      }
      result.success(null)
    } else if (call.method == "showSpecificCampaign") {
      if (activity != null) { 
        val campaignType = call.argument("campaignType") as String?
        if (campaignType != null) {
          if (campaignType == "attendance") {
            BuzzBooster.getInstance().showCampaign(activity!!, CampaignType.Attendance)
          } else if (campaignType == "referral") {
            BuzzBooster.getInstance().showCampaign(activity!!, CampaignType.Referral)
          }
        }
      }
      result.success(null)
    } else {
      result.notImplemented()
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
      activity = binding.activity
    }

    override fun onDetachedFromActivity() {
      activity = null
    }

    override fun onDetachedFromActivityForConfigChanges() {
      activity = null
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
      activity = binding.activity
    }
}
