import 'dart:convert';
import 'dart:io';
import 'package:buzz_booster_example/ServerMutator.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:http/http.dart' as http;
import 'Partner.dart';
import 'package:flutter/services.dart';

const String defaultAndroidAppKey = "307117684877774";
const String defaultiosAppKey = "279753136766115";
const String androidAppKeyKey = "androidAppKey";
const String androidAppKeyListKey = "androidAppKeyList";
const String iosAppKeyKey = "iosAppKey";
const String iosAppKeyListKey = "iosAppKeyList";
const String userIdKey = "userIdKey";
const String serverEnvironmentKey = "getServerEnvironmentKey";

class Repository {
  Future<SharedPreferences> _prefs = SharedPreferences.getInstance();

  Future<String> getAppKey() async {
    if (Platform.isAndroid) {
      return getAndroidAppKey();
    } else {
      return getiOSAppKey();
    }
  }

  Future<Environment> getServerEnvironment() async {
    final SharedPreferences prefs = await _prefs;
    final string = prefs.getString(serverEnvironmentKey);
    if (string == null) {
      return Future.value(Environment.Production);
    } else {
      final env = Environment.values.firstWhere((element) => element.toString() == string);
      return Future.value(env);
    }
  }

  Future<void> saveServerEnvironment(Environment environment) async {
    final SharedPreferences prefs = await _prefs;
    prefs.setString(serverEnvironmentKey, environment.toString());
  }

  Future<List<Partner>> getAppKeys() async {
    String url;
    Environment env = await getServerEnvironment();
    switch(env) {
      case Environment.Dev:
        url = "https://api-dev.buzzvil.com/buzzbooster/admin/manual/check-app-key";
        break;
      case Environment.Staging:
        url = "https://api-staging.buzzvil.com/buzzbooster/admin/manual/check-app-key";
        break;
      default:
        url = "https://api.buzzvil.com/buzzbooster/admin/manual/check-app-key";
        break;
    }
    final response = await http.get(Uri.parse(url));
    final body = utf8.decode(response.bodyBytes);
    return (jsonDecode(body) as List).map((value) => Partner.fromJson(value)).toList();
  }

  Future<String> getAndroidAppKey() async {
    final SharedPreferences prefs = await _prefs;
    return prefs.getString(androidAppKeyKey) ?? defaultAndroidAppKey;
  }

  Future<String> getiOSAppKey() async {
    final SharedPreferences prefs = await _prefs;
    return prefs.getString(iosAppKeyKey) ?? defaultiosAppKey;
  }

  Future<String> getUserId() async {
    final SharedPreferences prefs = await _prefs;
    return prefs.getString(userIdKey) ?? "Damon";
  }

  Future<void> saveAppKey(String appKey, String userId) async {
    final SharedPreferences prefs = await _prefs;
    if (Platform.isAndroid) {
      prefs.setString(androidAppKeyKey, appKey);
    } else {
      prefs.setString(iosAppKeyKey, appKey);
    }
    prefs.setString(userIdKey, userId);
  }

  Future<void> clearCursor(String appKey) async {
    try {
      await MethodChannel('com.buzzvil.dev/booster').invokeMethod('clearCursor', {
        "appKey": appKey
      });
    } catch (e) {
      print("fail to change");
    }
    return Future.value();
  }
}
