import 'package:flutter/widgets.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

const String viewType = 'CampaignEntryView';

class IOSCampaignEntryView extends StatelessWidget {
  final Map<String, dynamic> creationParams = <String, dynamic>{};

  IOSCampaignEntryView({Key? key, entryName}) : super(key: key) {
    creationParams["entry_name"] = entryName;
  }

  @override
  Widget build(BuildContext context) {
    return UiKitView(
        key: key,
        viewType: viewType,
        layoutDirection: TextDirection.ltr,
        creationParams: creationParams,
        creationParamsCodec: const StandardMessageCodec());
  }
}

class AndroidCampaignEntryView extends StatelessWidget {
  final Map<String, dynamic> creationParams = <String, dynamic>{};

  AndroidCampaignEntryView({Key? key, entryName}) {
    creationParams["entry_name"] = entryName;
  }

  @override
  Widget build(BuildContext context) {
    return AndroidView(
      key: key,
      viewType: viewType,
      layoutDirection: TextDirection.ltr,
      creationParams: creationParams,
      creationParamsCodec: const StandardMessageCodec(),
    );
  }
}

class NativeCampaignEntryView extends StatelessWidget {
  final String? entryName;

  const NativeCampaignEntryView({Key? key, this.entryName}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    switch (defaultTargetPlatform) {
      case TargetPlatform.android:
        return AndroidCampaignEntryView(key: key, entryName: entryName);
      case TargetPlatform.iOS:
        return IOSCampaignEntryView(key: key, entryName: entryName);
      default:
        throw UnsupportedError('Unsupported platform view');
    }
  }
}

class CampaignEntryView extends StatelessWidget {
  final Widget child;
  final double height;
  final double width;
  final String? entryName;

  const CampaignEntryView(
      {Key? key,
      required this.child,
      required this.width,
      required this.height,
      this.entryName})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Stack(children: [
      child,
      SizedBox(
          width: 100,
          height: 100,
          child: NativeCampaignEntryView(
            key: key,
            entryName: entryName,
          )),
    ]);
  }
}
