import 'dart:io';
import 'package:buzz_booster/buzz_booster.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'buzz_booster_platform_interface.dart';

class MethodChannelBuzzBooster extends BuzzBoosterPlatform {

  @visibleForTesting
  final methodChannel = const MethodChannel('buzz_booster');

  @override
  Future<void> setUserId(String? userId, ) async {
    return await methodChannel.invokeMethod<void>("setUserId", {
      "userId": userId,
    });
  }

  @override
  Future<void> init({required String androidAppKey,required  String iosAppKey}) async {
    return await methodChannel.invokeMethod<void>("init", {
      "androidAppKey": androidAppKey,
      "iosAppKey": iosAppKey,
    });
  }

  @override
  Future<void> startService() async {
    if (Platform.isIOS) {
      return await methodChannel.invokeMethod<void>("startService", {});
    } else {
      return await Future.value();
    }
  }

  @override
  Future<void> sendEvent(String eventName, Map<String, String>? eventValues, ) async {
    return await methodChannel.invokeMethod<void>("sendEvent", {
      "eventName": eventName,
      "eventValues": eventValues,
    });
  }

  @override
  Future<void> showInAppMessage() async {
    return await methodChannel.invokeMethod<void>("showInAppMessage", {
    });
  }

  @override
  Future<void> showCampaign() async {
    return await methodChannel.invokeMethod<void>("showCampaign", {
    });
  }

@override
  Future<void> showSpecificCampaign(CampaignType campaignType) async {
    return await methodChannel.invokeMethod<void>("showSpecificCampaign", {
      "campaignType":campaignType.name
    });
  }
}
