import 'package:flutter_test/flutter_test.dart';
import 'package:buzz_booster/buzz_booster.dart';
import 'package:buzz_booster/buzz_booster_platform_interface.dart';
import 'package:buzz_booster/buzz_booster_method_channel.dart';
import 'package:plugin_platform_interface/plugin_platform_interface.dart';

class MockBuzzBoosterPlatform 
    with MockPlatformInterfaceMixin
    implements BuzzBoosterPlatform {

  @override
  Future<void> setUserId(String? userId) => Future.value();

  @override
  Future<void> init({required String androidAppKey, required String iosAppKey}) => Future.value();

  @override
  Future<void> startService() => Future.value();

  @override
  Future<void> sendEvent(String eventName, Map<String, String>? eventValues) => Future.value();

  @override
  Future<void> showInAppMessage() => Future.value();

  @override
  Future<void> showCampaign() => Future.value();

  @override
  Future<void> showSpecificCampaign(CampaignType type) => Future.value();
}

void main() {
  final BuzzBoosterPlatform initialPlatform = BuzzBoosterPlatform.instance;

  test('$MethodChannelBuzzBooster is the default instance', () {
    expect(initialPlatform, isInstanceOf<MethodChannelBuzzBooster>());
  });

  test('setUserId', () async {
    BuzzBooster buzzBoosterPlugin = BuzzBooster();
    MockBuzzBoosterPlatform fakePlatform = MockBuzzBoosterPlatform();
    BuzzBoosterPlatform.instance = fakePlatform;
  
    await buzzBoosterPlugin.setUserId("Damon");
    await buzzBoosterPlugin.setUserId(null);
  });

  test('init', () async {
    BuzzBooster buzzBoosterPlugin = BuzzBooster();
    MockBuzzBoosterPlatform fakePlatform = MockBuzzBoosterPlatform();
    BuzzBoosterPlatform.instance = fakePlatform;
  
    await buzzBoosterPlugin.init(
      androidAppKey: "androidAppKey",
      iosAppKey: "iosAppKey"
    );
  });

  test('startService', () async {
    BuzzBooster buzzBoosterPlugin = BuzzBooster();
    MockBuzzBoosterPlatform fakePlatform = MockBuzzBoosterPlatform();
    BuzzBoosterPlatform.instance = fakePlatform;
  
    await buzzBoosterPlugin.startService();
  });

  test('sendEvent', () async {
    BuzzBooster buzzBoosterPlugin = BuzzBooster();
    MockBuzzBoosterPlatform fakePlatform = MockBuzzBoosterPlatform();
    BuzzBoosterPlatform.instance = fakePlatform;
  
    await buzzBoosterPlugin.sendEvent("eventName", {
      "eventValuesKey1":"eventValuesValue1"
    });
  });

  test('sendEventWithNoValues', () async {
    BuzzBooster buzzBoosterPlugin = BuzzBooster();
    MockBuzzBoosterPlatform fakePlatform = MockBuzzBoosterPlatform();
    BuzzBoosterPlatform.instance = fakePlatform;
  
    await buzzBoosterPlugin.sendEvent("eventName", null);
  });

  test('showInAppMessage', () async {
    BuzzBooster buzzBoosterPlugin = BuzzBooster();
    MockBuzzBoosterPlatform fakePlatform = MockBuzzBoosterPlatform();
    BuzzBoosterPlatform.instance = fakePlatform;
  
    await buzzBoosterPlugin.showInAppMessage();
  });

  test('showCampaign', () async {
    BuzzBooster buzzBoosterPlugin = BuzzBooster();
    MockBuzzBoosterPlatform fakePlatform = MockBuzzBoosterPlatform();
    BuzzBoosterPlatform.instance = fakePlatform;
  
    await buzzBoosterPlugin.showCampaign();
  });

  test('showSpecificCampaign', () async {
    BuzzBooster buzzBoosterPlugin = BuzzBooster();
    MockBuzzBoosterPlatform fakePlatform = MockBuzzBoosterPlatform();
    BuzzBoosterPlatform.instance = fakePlatform;
  
    await buzzBoosterPlugin.showSpecificCampaign(CampaignType.attendance);
  });
}
